﻿//
// Copyright (c) 2008-2010, MindFusion LLC - Bulgaria.
//

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;


namespace MindFusion.Scheduling.Silverlight.Samples.CS.CellPresenterSample
{
	/// <summary>
	/// A CellPresenter which also shows an indication if there are invisible items
	/// in the cell.
	/// </summary>
	public class AdvancedCellPresenter : CellPresenter
	{
		public AdvancedCellPresenter(Calendar calendar)
			: base(calendar)
		{
			DefaultStyleKey = typeof(AdvancedCellPresenter);
			StyleKeyWithHeader = "AdvancedCellStyle";

			calendar.ItemCreated += (s, e) =>
			{
				UpdateHiddenIconState();
			};
			calendar.ItemDeleted += (s, e) =>
			{
				UpdateHiddenIconState();
			};
			calendar.ItemModified += (s, e) =>
			{
				UpdateHiddenIconState();
			};
			calendar.ItemModifying += (s, e) =>
			{
				UpdateHiddenIconState();
			};

			this.SizeChanged += (s, e) =>
			{
				UpdateHiddenIconState();
			};
		}

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();

			contentRow = GetTemplateChild("ContentRow") as RowDefinition;
		}

		RowDefinition contentRow;

		private void UpdateHiddenIconState()
		{
			if (contentRow == null)
				return;

			var availableSpace = contentRow.ActualHeight;
			var itemHeight = Calendar.ItemSettings.Size;
			var itemSpacing = Calendar.ItemSettings.Padding;

			availableSpace -= itemSpacing.Bottom;
			int i = 0;
			double space = availableSpace;
			while (space > itemHeight)
			{
				i++;
				space -= (itemHeight + itemSpacing.Top + itemSpacing.Bottom);
			}
			int visibleItems = i;// (int)Math.Floor(availableSpace / (itemHeight + itemSpacing.Top + itemSpacing.Bottom));

			var items = Calendar.Schedule.GetAllItems(
				this.StartTime, this.EndTime - TimeSpan.FromTicks(1));
			HasHiddenItemsVisibility = items.Count > visibleItems ? Visibility.Visible : Visibility.Collapsed;
		}

		/// <summary>
		/// Gets or sets a value indicating the visibility of the hidden items icon.
		/// </summary>
		public Visibility HasHiddenItemsVisibility
		{
			get { return (Visibility)GetValue(HasHiddenItemsVisibilityProperty); }
			set { SetValue(HasHiddenItemsVisibilityProperty, value); }
		}

		public static readonly DependencyProperty HasHiddenItemsVisibilityProperty =
			DependencyProperty.Register("HasHiddenItemsVisibility", typeof(Visibility),
			typeof(AdvancedCellPresenter), new PropertyMetadata(Visibility.Collapsed));
	}
}
